@@ -46,6 +46,8 @@ DEFAULT_CLIENT_NO_INTERMEDIATE_CERTS = {
     'certfile': os.path.join(CERTS_PATH, 'client_no_intermediate.pem'),
     'keyfile': os.path.join(CERTS_PATH, 'client_intermediate.key'),
 }
+PASSWORD_KEYFILE = os.path.join(CERTS_PATH, 'server_password.key')
+PASSWORD_CLIENT_KEYFILE = os.path.join(CERTS_PATH, 'client_password.key')
 NO_SAN_CERTS = {
     'certfile': os.path.join(CERTS_PATH, 'server.no_san.crt'),
     'keyfile': DEFAULT_CERTS['keyfile']
@@ -58,11 +60,16 @@ IPV6_ADDR_CERTS = {
     'certfile': os.path.join(CERTS_PATH, 'server.ipv6addr.crt'),
     'keyfile': os.path.join(CERTS_PATH, 'server.ipv6addr.key'),
 }
+IPV6_SAN_CERTS = {
+    'certfile': os.path.join(CERTS_PATH, 'server.ipv6_san.crt'),
+    'keyfile': DEFAULT_CERTS['keyfile']
+}
 DEFAULT_CA = os.path.join(CERTS_PATH, 'cacert.pem')
 DEFAULT_CA_BAD = os.path.join(CERTS_PATH, 'client_bad.pem')
 NO_SAN_CA = os.path.join(CERTS_PATH, 'cacert.no_san.pem')
 DEFAULT_CA_DIR = os.path.join(CERTS_PATH, 'ca_path_test')
 IPV6_ADDR_CA = os.path.join(CERTS_PATH, 'server.ipv6addr.crt')
+IPV6_SAN_CA = os.path.join(CERTS_PATH, 'server.ipv6_san.crt')
 COMBINED_CERT_AND_KEY = os.path.join(CERTS_PATH, 'server.combined.pem')
 
 
@@ -226,15 +233,16 @@ def bind_sockets(port, address=None, family=socket.AF_UNSPEC, backlog=128,
 
 
 def run_tornado_app(app, io_loop, certs, scheme, host):
+    assert io_loop == tornado.ioloop.IOLoop.current()
+
     # We can't use fromtimestamp(0) because of CPython issue 29097, so we'll
     # just construct the datetime object directly.
     app.last_req = datetime(1970, 1, 1)
 
     if scheme == 'https':
-        http_server = tornado.httpserver.HTTPServer(app, ssl_options=certs,
-                                                    io_loop=io_loop)
+        http_server = tornado.httpserver.HTTPServer(app, ssl_options=certs)
     else:
-        http_server = tornado.httpserver.HTTPServer(app, io_loop=io_loop)
+        http_server = tornado.httpserver.HTTPServer(app)
 
     sockets = bind_sockets(None, address=host)
     port = sockets[0].getsockname()[1]
@@ -268,7 +276,7 @@ if __name__ == '__main__':
     from .testcase import TestingApp
     host = '127.0.0.1'
 
-    io_loop = tornado.ioloop.IOLoop()
+    io_loop = tornado.ioloop.IOLoop.current()
     app = tornado.web.Application([(r".*", TestingApp)])
     server, port = run_tornado_app(app, io_loop, None,
                                    'http', host)
